@@ -277,7 +277,7 @@ struct ibv_mr *bnxt_re_reg_dmabuf_mr(struct ibv_pd *ibvpd, uint64_t start, size_
 		return NULL;
 
 	if (ibv_cmd_reg_dmabuf_mr(ibvpd, start, len, iova, fd,
-				  access, &mr->vmr)) {
+				  access, &mr->vmr, NULL)) {
 		free(mr);
 		return NULL;
 	}
@@ -1180,6 +1180,8 @@ static int bnxt_re_get_rq_slots(struct bnxt_re_dev *rdev, uint8_t qpmode,
 	return slots;
 }
 
+#define BNXT_VAR_MAX_SLOT_ALIGN 256
+
 static int bnxt_re_get_sq_slots(struct bnxt_re_dev *rdev,
 				uint8_t qpmode, uint32_t nswr,
 				uint32_t nsge, uint32_t ils, uint32_t *esize)
@@ -1216,6 +1218,8 @@ static int bnxt_re_get_sq_slots(struct bnxt_re_dev *rdev,
 	if (esize)
 		*esize = wqe_size;
 	slots = (nswr * wqe_size) / stride;
+	if (qpmode == BNXT_RE_WQE_MODE_VARIABLE)
+		slots = align(slots, BNXT_VAR_MAX_SLOT_ALIGN);
 	return slots;
 }
 
@@ -1523,6 +1527,8 @@ struct ibv_qp *bnxt_re_create_qp(struct ibv_pd *ibvpd,
 	req.qpsva = (uintptr_t)qp->jsqq->hwque->va;
 	req.qprva = qp->jrqq ? (uintptr_t)qp->jrqq->hwque->va : 0;
 	req.qp_handle = (uintptr_t)qp;
+	if (qp->qpmode == BNXT_RE_WQE_MODE_VARIABLE)
+		req.sq_slots = qattr[BNXT_RE_QATTR_SQ_INDX].slots;
 
 	if (ibv_cmd_create_qp(ibvpd, &qp->ibvqp, attr, &req.ibv_cmd, sizeof(req),
 			      &resp.ibv_resp, sizeof(resp)))
@@ -2230,8 +2236,9 @@ struct ibv_srq *bnxt_re_create_srq(struct ibv_pd *ibvpd,
 				   struct ibv_srq_init_attr *attr)
 {
 	struct bnxt_re_context *cntx = to_bnxt_re_context(ibvpd->context);
+	struct bnxt_re_mmap_info minfo = {};
+	struct ubnxt_re_srq_resp resp = {};
 	struct bnxt_re_qattr qattr = {};
-	struct ubnxt_re_srq_resp resp;
 	struct ubnxt_re_srq req;
 	struct bnxt_re_srq *srq;
 	void *mem;
@@ -2266,7 +2273,19 @@ struct ibv_srq *bnxt_re_create_srq(struct ibv_pd *ibvpd,
 	srq->cap.max_sge = attr->attr.max_sge;
 	srq->cap.srq_limit = attr->attr.srq_limit;
 	srq->arm_req = false;
-
+	if (resp.comp_mask & BNXT_RE_SRQ_TOGGLE_PAGE_SUPPORT) {
+		minfo.type = BNXT_RE_SRQ_TOGGLE_MEM;
+		minfo.res_id = resp.srqid;
+		ret = bnxt_re_get_toggle_mem(ibvpd->context, &minfo, &srq->mem_handle);
+		if (ret)
+			goto fail;
+		srq->toggle_map = mmap(NULL, minfo.alloc_size, PROT_READ,
+				       MAP_SHARED, ibvpd->context->cmd_fd,
+				       minfo.alloc_offset);
+		if (srq->toggle_map == MAP_FAILED)
+			goto fail;
+		srq->toggle_size = minfo.alloc_size;
+	}
 	return &srq->ibvsrq;
 fail:
 	bnxt_re_free_mem(mem);
@@ -2300,6 +2319,8 @@ int bnxt_re_destroy_srq(struct ibv_srq *ibvsrq)
 	if (ret)
 		return ret;
 
+	if (srq->toggle_map)
+		munmap(srq->toggle_map, srq->toggle_size);
 	mem = srq->mem;
 	bnxt_re_free_mem(mem);
 	return 0;
