@@ -510,12 +510,15 @@ static void parse_for_ud_qp(struct hns_roce_v2_cqe *cqe, struct ibv_wc *wc)
 }
 
 static void parse_cqe_for_srq(struct hns_roce_v2_cqe *cqe, struct ibv_wc *wc,
-			      struct hns_roce_srq *srq)
+			      struct hns_roce_srq *srq,
+			      struct hns_roce_qp *hr_qp)
 {
 	uint32_t wqe_idx;
 
 	if (hr_reg_read(cqe, CQE_CQE_INLINE))
 		handle_recv_cqe_inl_from_srq(cqe, srq);
+	else if (hr_qp->verbs_qp.qp.qp_type == IBV_QPT_UD)
+		parse_for_ud_qp(cqe, wc);
 
 	wqe_idx = hr_reg_read(cqe, CQE_WQE_IDX);
 	wc->wr_id = srq->wrid[wqe_idx & (srq->wqe_cnt - 1)];
@@ -531,13 +534,13 @@ static void parse_cqe_for_resp(struct hns_roce_v2_cqe *cqe, struct ibv_wc *wc,
 	wc->wr_id = wq->wrid[wq->tail & (wq->wqe_cnt - 1)];
 	++wq->tail;
 
-	if (hr_qp->verbs_qp.qp.qp_type == IBV_QPT_UD)
-		parse_for_ud_qp(cqe, wc);
-
 	if (hr_reg_read(cqe, CQE_CQE_INLINE))
 		handle_recv_cqe_inl_from_rq(cqe, hr_qp);
 	else if (hr_reg_read(cqe, CQE_RQ_INLINE))
 		handle_recv_rq_inl(cqe, hr_qp);
+	else if (hr_qp->verbs_qp.qp.qp_type == IBV_QPT_UD)
+		parse_for_ud_qp(cqe, wc);
+
 }
 
 static void parse_cqe_for_req(struct hns_roce_v2_cqe *cqe, struct ibv_wc *wc,
@@ -669,7 +672,7 @@ static int parse_cqe_for_cq(struct hns_roce_context *ctx, struct hns_roce_cq *cq
 			return V2_CQ_POLL_ERR;
 
 		if (srq)
-			parse_cqe_for_srq(cqe, wc, srq);
+			parse_cqe_for_srq(cqe, wc, srq, cur_qp);
 		else
 			parse_cqe_for_resp(cqe, wc, cur_qp);
 	}
@@ -1223,7 +1226,7 @@ static int set_rc_wqe(void *wqe, struct hns_roce_qp *qp, struct ibv_send_wr *wr,
 
 	hr_reg_write_bool(wqe, RCWQE_CQE,
 			  !!(wr->send_flags & IBV_SEND_SIGNALED));
-	hr_reg_write_bool(wqe, RCWQE_FENCE,
+	hr_reg_write_bool(wqe, RCWQE_SO,
 			  !!(wr->send_flags & IBV_SEND_FENCE));
 	hr_reg_write_bool(wqe, RCWQE_SE,
 			  !!(wr->send_flags & IBV_SEND_SOLICITED));
@@ -1272,7 +1275,7 @@ int hns_roce_u_v2_post_send(struct ibv_qp *ibvqp, struct ibv_send_wr *wr,
 	struct hns_roce_context *ctx = to_hr_ctx(ibvqp->context);
 	struct hns_roce_qp *qp = to_hr_qp(ibvqp);
 	struct hns_roce_sge_info sge_info = {};
-	struct hns_roce_rc_sq_wqe *wqe;
+	struct hns_roce_rc_sq_wqe *wqe = NULL;
 	struct ibv_qp_attr attr = {};
 	unsigned int wqe_idx, nreq;
 	int ret;
@@ -1288,15 +1291,15 @@ int hns_roce_u_v2_post_send(struct ibv_qp *ibvqp, struct ibv_send_wr *wr,
 	sge_info.start_idx = qp->next_sge; /* start index of extend sge */
 
 	for (nreq = 0; wr; ++nreq, wr = wr->next) {
-		if (hns_roce_v2_wq_overflow(&qp->sq, nreq,
-					    to_hr_cq(qp->verbs_qp.qp.send_cq))) {
-			ret = ENOMEM;
+		if (wr->num_sge > (int)qp->sq.max_gs) {
+			ret = qp->sq.max_gs > 0 ? EINVAL : EOPNOTSUPP;
 			*bad_wr = wr;
 			goto out;
 		}
 
-		if (wr->num_sge > qp->sq.max_gs) {
-			ret = EINVAL;
+		if (hns_roce_v2_wq_overflow(&qp->sq, nreq,
+					    to_hr_cq(qp->verbs_qp.qp.send_cq))) {
+			ret = ENOMEM;
 			*bad_wr = wr;
 			goto out;
 		}
